<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl"
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:func="http://oxygenxml.com/samples/books"
    exclude-result-prefixes="xs xd func"
    xpath-default-namespace="http://oxygenxml.com/samples/books"
    version="2.0">
    
    <xd:doc scope="stylesheet">
        <xd:desc>
            <xd:p><xd:b>Created on:</xd:b> May 8, 2013</xd:p>
            <xd:p><xd:b>Author:</xd:b> Octavian Nadolu</xd:p>
            <xd:p>Generates an HTML file containing a table with books.</xd:p>
        </xd:desc>
    </xd:doc>
    
    <xd:doc>
        <xd:desc>
            <xd:b>Specifies if the books should be grouped by technology.</xd:b>
        </xd:desc>
    </xd:doc>
    <xsl:param name="groupByTechnology" select="false()"/>
    
    <xd:doc>
        <xd:desc>
            <xd:b>Depending on the value of the parameter an index of the books will be created or
                not.</xd:b>
        </xd:desc>
    </xd:doc>
    <xsl:param name="generateIndex" select="true()"/>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a table with books.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="books">
        <html>
            <head>
                <title>Recommended Books</title>
            </head>
            <body>
                <h1>The list of recommended books</h1>
                
                <xsl:if test="$generateIndex">
                    <xsl:copy-of select="func:generateIndex(book)"/>
                </xsl:if>
                
                <xsl:choose>
                    <xsl:when test="$groupByTechnology">
                        <!-- Generates a table for each group of books that have the same technology.-->
                        <xsl:for-each-group select="book" group-by="@technology">
                            
                            <xsl:call-template name="generateCategoryTitle">
                                <xsl:with-param name="category" select="@technology"/>
                            </xsl:call-template>
                            
                            <xsl:variable name="books" select="current-group()"/>
                            <!-- A table is create for each group. -->
                            <table cellspacing="12px">
                                <xsl:apply-templates select="$books"/>
                            </table>
                        </xsl:for-each-group>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- Generates a table with books, in the order they are in the input XML document. -->
                        <table cellspacing="12px">
                            <xsl:apply-templates select="book"/>
                        </table>
                    </xsl:otherwise>
                </xsl:choose>
            </body>
        </html>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates an index for the given books.</xd:p>
        </xd:desc>
        <xd:param name="books">The books to generate the index for.</xd:param>
    </xd:doc>
    <xsl:function name="func:generateIndex">
        <xsl:param name="books"/>
        <!-- Index  -->
        <xsl:for-each select="$books">
            <xsl:call-template name="createIndexItem">
                <xsl:with-param name="bookAnchor" select="@anchor"/>
                <xsl:with-param name="bookName" select="title"/>
                <xsl:with-param name="bookAuthor" select="author"/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:function>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates the a title for the given book category.</xd:p>
        </xd:desc>
        <xd:param name="category">The name of the book category.</xd:param>
    </xd:doc>
    <xsl:template name="generateCategoryTitle">
        <xsl:param name="category"/>
        <h2>
            <xsl:value-of select="$category"/>
        </h2>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a link for a book the index.</xd:p>
        </xd:desc>
        <xd:param name="bookAnchor">The anchor of the reference.</xd:param>
        <xd:param name="bookName">The text displayed for the link.</xd:param>
        <xd:param name="bookAuthor">The author of the book.</xd:param>
    </xd:doc>
    <xsl:template name="createIndexItem">
        <xsl:param name="bookAnchor"/>
        <xsl:param name="bookName"/>
        <xsl:param name="bookAuthor"/>
        <p>
            <a href="{concat('#', $bookAnchor)}" title="{$bookName}">
                <xsl:value-of select="$bookName"/>
            </a>
            <i> - by <xsl:value-of select="$bookAuthor"/></i>
        </p>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Creates a row with two columns for each book.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="book">
        <!-- Book -->
        <tr>
            <td valign="top">
                <!--Image-->
                <xsl:apply-templates select="image"/>
            </td>
            <td>
                <a name="{@anchor}"/>
                <!-- Title with link -->
                <xsl:variable name="title" select="title"/>
                <xsl:variable name="titleURI" select="@targeturi"/>
                
                <xsl:call-template name="linkTitle">
                    <xsl:with-param name="titleUri" select="$titleURI"/>
                    <xsl:with-param name="title" select="$title"/>
                </xsl:call-template>
                <!-- Author -->
                <p>
                    <i>by 
                        <xsl:value-of select="author"/>
                    </i>
                </p>
                <!-- Book info -->
                <b>Paperback: </b><xsl:value-of select="paperback"/> | 
                <b>Publisher: </b><xsl:value-of select="publisher"/> | 
                <b>Isbn: </b><xsl:value-of select="isbn"/>
                <!-- Description -->
                <xsl:apply-templates select="description"/>
            </td>
        </tr>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Creates an image element.</xd:p>
        </xd:desc>
    </xd:doc>
    <xsl:template match="image">
        <img src="{@src}" alt="{@description}" width="110" height="140"/>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>
            <xd:p>Generates a title with link</xd:p>
        </xd:desc>
        <xd:param name="titleUri">The URI of the title</xd:param>
        <xd:param name="title">The value of the title</xd:param>
    </xd:doc>
    <xsl:template name="linkTitle">
        <xsl:param name="titleUri"/>
        <xsl:param name="title"/>
        <h3>
            <a href="{$titleUri}" title="{$title}">
                <xsl:value-of select="$title"/>
            </a>
        </h3>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>Creates a paragraph element.</xd:desc>
    </xd:doc>
    <xsl:template match="para">
        <p>
            <xsl:apply-templates/>
        </p>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>Creates an italic element.</xd:desc>
    </xd:doc>
    <xsl:template match="italic">
        <i>
            <xsl:apply-templates/>
        </i>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>Creates a list element.</xd:desc>
    </xd:doc>
    <xsl:template match="list">
        <ul><xsl:apply-templates/></ul>
    </xsl:template>
    
    <xd:doc>
        <xd:desc>Creates a list item element.</xd:desc>
    </xd:doc>
    <xsl:template match="element">
        <li>
            <xsl:apply-templates/>
        </li>
    </xsl:template>
</xsl:stylesheet>
